@@ -1,12 +1,13 @@
 .First.lib <- function(lib=NULL, pkg=ks)
 {
   library.dynam("ks", pkg, lib)
-  cat("ks version 1.3.5 (2005) \n")
+  cat("ks version 1.4.0  (2006) \n")
+  require(mvtnorm)
+  require(misc3d)
+  require(rgl)
+  #require(feature)
 }  
 
-require(mvtnorm)
-require(misc3d)
-require(rgl)
 
 ###############################################################################
 # Basic operators and functions
@@ -403,19 +404,35 @@ permute.mat <- function(order)
     q
 }
 
-##These function are courtesy of Matt Wand 2005
 
-spheres3d.rh <- function(x,y,z,radius,...)
+
+########## R functions: rhcs.xxxx ##########
+
+# Transformations of rgl functions for compatibility
+# with misc3d function
+
+# Last changed: 15 SEP 2005
+
+
+rhcs.spheres3d <- function(x,y,z,radius,...)
   return(spheres3d(x,z,-y,radius,...))
 
-texts3d.rh <- function(x,y,z,text,...)
+rhcs.texts3d <- function(x,y,z,text,...)
   return(texts3d(x,z,-y,text,...))
 
-triangles3d.rh <- function(x,y,z,...)
+rhcs.triangles3d <- function(x,y,z,...)
   return(triangles3d(x,z,-y,...))
 
-quads3d.rh <- function(x,y,z,...)
-  return(quads3d(x,z,-y,...))
-
-points3d.rh <- function(x,y,z,...)
+rhcs.points3d <- function(x,y,z,...)
   return(points3d(x,z,-y,...))
+
+rhcs.quads3d <- function(x,y,z,...)
+  return(quads3d(x,z,-y,...))
+    
+rhcs.clear3d <- function(type="shapes")
+{
+  rgl.viewpoint(theta=135)
+  return(rgl.clear(type))
+}
+
+########## End of rhcs.xxxx ##########
